//=============================================================================
//  TMidiPlayback Implementation
//=============================================================================
#include <mcPlayback.h>
#include <gtk/gtk.h>

bool PlayTimerCallback( gpointer data );
bool GuiTimerCallback( gpointer data );

const int DEVICE = 0;  // TEMPORARY

//=============================================================================
// constructor TMidiPlayback
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiPlayback::TMidiPlayback( TMidiSequence* aSequence, TMidiMapper* aMapper )
{
  fSequence = aSequence;
  fMapper = aMapper;
  fIO = fMapper->GetIO();
  fEventList = 0;
  fTempoList = 0;
  fPlayTimerID = -1;
  fGuiTimerID = -1;
}

//=============================================================================
// destructor TMidiPlayback
// 05.03.2001 Adrian Meyer
//=============================================================================
TMidiPlayback::~TMidiPlayback( void )
{
  Stop();
}

//=============================================================================
// playback timer
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::PlayTimer( void )
{

	fPlayPosition = fIO->GetTime();

  for ( i=0; i<fTrackCount; i++ )                     // check playable events
    if ( fEventList[i] )
    {                                                 // check offset
      while ( fEventList[i]->GetOffset() < fPlayPosition )
      {
        PlayEvent( fEventList[i] );                   // play the event
				fEventList[i] = (TMidiEvent*)(fEventList[i]->Next);          // switch to next
        if ( !fEventList[i] ) break;                  // last event
      }
    }

  while ( fTempoList->Offset < fPlayPosition )        // check tempo
  {
    if ( fTempoList->Next )
      fTempoList = fTempoList->Next;
    else
      break;
  }


  if ( fPlayPosition > fEndPosition )                 // end reached
  {
    fPlayPosition = fEndPosition;                     // reset to end position
    Stop();                                           // stop playback
  }
}

//=============================================================================
// play midi event
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::PlayEvent( TMidiEvent* aEvent )
{
	fIO->Write(DEVICE, aEvent);
}


//=============================================================================
// gui timer
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::GuiTimer( void )
{
  fSequence->SetPosition( fPlayPosition );            // update TMidiSequence
}

//=============================================================================
// initialize start
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::Start( void )
{
  if ( fPlayTimerID != -1 ) return;                   // already running
  if ( fSequence->GetTrackCount() < 1 ) return;

	// Set the output device from 1st track
  fIO->SetOutputDevice( fMapper->GetOutputDeviceID( fSequence->GetTrack(1)->GetPort() ));
	// Set the timebase first
	fIO->SetTimebase(fSequence->GetDeltaTimesType());

  fTempoList = fSequence->GetTempo(0);
  fTrackCount = fSequence->GetTrackCount(); // list with playable events
  fEventList = new TMidiEvent*[ fSequence->GetTrackCount() ];
  for ( i = 0; i<fTrackCount; i++ )
  {
		TMidiTrack *trk = fSequence->GetTrack(i);
    fEventList[i] = trk->GetEvent(0);

    if (trk->GetInstrumentPatch() >= 0 )     // send initial program changes
    {
			TMidiEvent *ev = new TMidiEvent(trk, 0);
			ev->SetData1(cESProgramChange | (trk->GetChannel() - 1));
			ev->SetData2(trk->GetInstrumentPatch());
			fIO->Write(DEVICE, ev);
			delete ev;
    }
  }

  fPlayPosition = fSequence->GetPosition();           // grab current position
  fEndPosition = fSequence->GetMaxOffset();           // grab max offset
  fPlayRest = 0;
                                                      // reset to 0 on end
  if ( fPlayPosition == fEndPosition ) fPlayPosition = 0;

  fPlayTimerID = gtk_timeout_add( 10, (GtkFunction)PlayTimerCallback, this );
  fGuiTimerID = gtk_timeout_add( 80, (GtkFunction)GuiTimerCallback, this );
  CallMCMessage( this, cMCPlayback, cMMPlaybackStart, "playback started");

	fIO->Start();
}

//=============================================================================
// stop playback
// 05.03.2001 Adrian Meyer
//=============================================================================
void TMidiPlayback::Stop( void )
{
  if ( fPlayTimerID == -1 ) return;                   // not running

  gtk_timeout_remove( fPlayTimerID );
  gtk_timeout_remove( fGuiTimerID );
  fPlayTimerID = -1;
  fGuiTimerID = -1;
  fIO->Stop();
  delete fEventList;

  fSequence->SetPosition( fPlayPosition );
  CallMCMessage( this, cMCPlayback, cMMPlaybackStop, "playback stoped");
}

//=============================================================================
// playback timer called
// 05.03.2001 Adrian Meyer
//=============================================================================
bool PlayTimerCallback(gpointer data)
{
  TMidiPlayback* playback = ((TMidiPlayback*)data );
  playback->PlayTimer();
  return true;
}

//=============================================================================
// update gui during playback
// 05.03.2001 Adrian Meyer
//=============================================================================
bool GuiTimerCallback(gpointer data)
{
  TMidiPlayback* playback = ((TMidiPlayback*)data );
  playback->GuiTimer();
  return true;
}
