//
//	INI File routines using the STL.  
//
//	copyright August 18, 1999 by Robert Kesterson
//
//	License:  
//			    You may use this code for any project you wish, 
//				whether commercial or not, provided that 
//				the following conditions are met:
//				(a) you drop me a line (email robertk@robertk.com) to let me know
//				     you're using it, and 
//				(b) give me a mention in the credits someplace
//
//
//	Notes:
//
//	INI keys *are* case sensitive in this implementation.
//  Lines in the INI that are > MAX_INI_LINE characters will not process correctly.  
//  (Default is MAX_INI_LINE of 500, but you can change it if you need more...)
//	
//	define TEST_INI to build a little test program.  Typical command line:
//	g++ -DTEST_INI -O2 -Wall -o testini stlini.cpp
//
//  Provides the following functionality:
//
//		Loads an INI file into a single nested hash, such that for each named section
//		in the INI file, you can use the name of the section to retrieve a INISection of all
//		the key/value pairs in the section.  Within the hash of key/value pairs, you just
//		use the key name to retrieve the value. 
//
//		Provides the capability to load and save the INI file to disk files.
//	
//		To load/save the INI files, use the following:
//
//	 		INIFile LoadIni(const char *filename)
//			void SaveIni(INIFile &theINI, const char *filename)
//
//		To get/set individual keys, use the following:
//
//			string GetIniSetting(INIFile &theINI, const char *section, const char *key)
//			void PutIniSetting(INIFile &theINI, const char *section, const char *key, const char *value)
//
//  Typical usage is illustrated by the test main() function below.  
//

#include <stdio.h>
#include "stlini.h"

string GetIniSetting(INIFile &theINI, const char *section, const char *key, const char *defaultval)
{
	string result(defaultval);

	INIFile::iterator INISection = theINI.find(string(section));
	INISection::iterator apair = INISection->second.find(string(key));
	if(apair != INISection->second.end())
		result = apair->second;
	return result;
}

void PutIniSetting(INIFile &theINI, const char *section, const char *key, const char *value)
{     
	INIFile::iterator iniSection;
	INISection::iterator apair;
	
	if((iniSection = theINI.find(section)) == theINI.end())
	{
		// no such section?  Then add one..
		INISection newsection;
		if(key)
			newsection.insert(	pair<string, string> (key, value) );
		theINI.insert( pair<string, INISection> (section, newsection) );
	}
	else if(key)
	{	// found section, make sure key isn't in there already, 
		// if it is, just drop and re-add
		apair = iniSection->second.find(key);
		if(apair != iniSection->second.end())
			iniSection->second.erase(apair);
		iniSection->second.insert( pair<string, string> (key, value) );
	}
}

INIFile LoadIni(const char *filename)
{
	INIFile theINI;
	char  *value, *temp;
	string section;
	char buffer[MAX_INI_LINE+1];
	
	if(FILE *file = fopen(filename, "r"))
	{
		while(!feof(file))
		{
			buffer[0] = buffer[MAX_INI_LINE] = '\0';
			fgets(buffer, MAX_INI_LINE, file);
			if((temp = strchr(buffer, '\n')))
				*temp = '\0';		// cut off at newline
			if((buffer[0] == '[') && (temp = strrchr(buffer, ']')))
			{     // if line is like -->   [section name]
				*temp = '\0';   // chop off the trailing ']';
				section = &buffer[1];
				PutIniSetting(theINI, &buffer[1]);	 // start new section
			}
			else if(buffer[0] && (value = strchr(buffer, '=')))
			{
				*value++ = '\0'; // assign whatever follows = sign to value, chop at "=" 
				PutIniSetting(theINI, section.c_str(), buffer, value); 	// and add both sides to INISection
			}
			else if(buffer[0])
				PutIniSetting(theINI, section.c_str(), buffer, ""); 	// must be a comment or something
		}
		fclose(file);
	}
	return theINI;
}

void SaveIni(INIFile &theINI, const char *filename)
{
	FILE *file = fopen(filename, "w");
	if(!file)
		return;
	
	// just iterate the hashes and values and dump them to a file.
	INIFile::iterator section= theINI.begin();
	while(section != theINI.end())
	{
		if(section->first > "") 
			fprintf(file, "\n[%s]\n", section->first.c_str());
		INISection::iterator pair = section->second.begin();
	
		while(pair != section->second.end())
		{
			fprintf(file, (pair->second > "") ? "%s=%s\n" : "%s\n", pair->first.c_str(), pair->second.c_str());
			pair++;
		}
		section++;
	}
	fclose(file);
}


#ifdef TEST_INI

void DumpIni(INIFile &ini)
{
	// This is essentially SaveIni() except it just dumps to stdout
	INIFile::iterator section= ini.begin();
	while(section != ini.end())
	{
		printf("\n[%s]\n", section->first.c_str());
		INISection sectionvals = section->second;
		INISection::iterator pair = sectionvals.begin();
	
		while(pair != sectionvals.end())
		{
			printf("%s=%s\n", pair->first.c_str(),
			pair->second.c_str());
			pair++;
		}
		section++;
	}
}

int main(int argc, char **argv)
{
	// read an INI.  If file doesn't exist, that's OK.
	INIFile ini = LoadIni("test.ini");
	if(ini.size())
	{
		// Note that existing INIs will be added to, though if any of the keys
		// listed below already exist, this program will modify them.
		printf("About to modify test.ini, which presently contains:\n");
		DumpIni(ini);
	}
   
	printf("\nLoading INI with the following information, plus comments\n\n");
	printf("[Favorites]\ncolor=blue\nfood=pizza\nbeer=homebrew\n\n");
	printf("[Computing]\nOperating System=Linux\nToolkit=FLTK\nComment=Now isn't this fun?\n\n");

	PutIniSetting(ini, "", "; This is a comment about the whole INI file");
	PutIniSetting(ini, "Favorites", "; This is a list of favorites");
	PutIniSetting(ini, "Favorites", "color", "blue");
	PutIniSetting(ini, "Favorites", "food", "pizza");
	PutIniSetting(ini, "Favorites", "beer", "homebrew");
	PutIniSetting(ini, "Computing", "; Information about computing preferences");
	PutIniSetting(ini, "Computing", "Operating System", "Linux");
	PutIniSetting(ini, "Computing", "Toolkit", "FLTK");
	PutIniSetting(ini, "Computing", "Comment", "This will be replaced in next line.");
	PutIniSetting(ini, "Computing", "Comment", "Now isn't this fun?");

	printf("\nINI Ready, saving to disk\n\n");
	SaveIni(ini, "test.ini");

	printf("Loading from disk to verify.\n\n");
	INIFile ini2 = LoadIni("test.ini");
	
	printf("Contents of ini just read\n\n");
	DumpIni(ini2);
	
	printf("\nChecking single value for section Computing, key Comment:\n");
	printf("Value is: %s\n", GetIniSetting(ini2, "Computing", "Comment").c_str());
	
	printf("\nChecking unset value for section Computing, \nkey Distribution, with default of \"RedHat\"\n");
	printf("Value is: %s\n\nDone\n\n", GetIniSetting(ini2, "Computing", "Distribution",  "RedHat").c_str());
	exit(0);
}
#endif


